@@ -1,4 +1,4 @@
-ClusterMCC=function(PriorCls,CurrentCls){
+ClusterMCC=function(PriorCls,CurrentCls,Force=TRUE){
  
   if (!requireNamespace('yardstick',quietly = TRUE)) {
     message(
@@ -15,6 +15,18 @@ ClusterMCC=function(PriorCls,CurrentCls){
   PriorCls[!is.finite(PriorCls)]=9999
   CurrentCls[!is.finite(CurrentCls)]=9999
   
+  if(isTRUE(Force)){
+    u1=sort(unique(PriorCls))
+    u2=sort(unique(CurrentCls))
+    s1=setdiff(u1,u2)
+    s2=setdiff(u2,u1)
+    ss=unique(c(s1,s2))
+    if(length(ss)>0){
+      message("ClusterMCC: No.of Clusters in PriorCls does not equal CurrentCls. Adding missing unique label(s).")
+      PriorCls=c(PriorCls,ss)
+      CurrentCls=c(CurrentCls,ss)
+    }
+  }
   CurrentClsFactor=as.factor(CurrentCls)
   PriorClsFactor=as.factor(PriorCls)
   if(length(levels(CurrentClsFactor))!=length(levels(PriorClsFactor))){
