@@ -1,4 +1,5 @@
-DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRUE,DistancesMethod=NULL,PlotTree=FALSE,PlotMap=FALSE,PlotIt=FALSE,Parallel=FALSE){
+DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRUE,DistancesMethod=NULL,
+                                   PlotTree=FALSE,PlotMap=FALSE,PlotIt=FALSE,Parallel=FALSE){
   # INPUT
   # DataOrDistances[1:n,1:d]    Either nonsymmetric [1:n,1:d] datamatrix of n cases and d features or
   #                             symmetric [1:n,1:n] distance matrix
@@ -34,15 +35,11 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
   DataPoints=NULL  # Init in case of no plotting
   Cls = rep(1, nrow(DataOrDistances))#init
   
-  if (!requireNamespace('DatabionicSwarm',quietly = TRUE)){
-    message(
-      'Subordinate clustering package (DatabionicSwarm) is missing. No computations are performed.
-            Please install the package which is defined in "Suggests".'
-    )
-    return(
-      list(
-        Cls = Cls,
-        Object = "Subordinate clustering package (DatabionicSwarm) is missing.
+  if(!requireNamespace('DatabionicSwarm',quietly = TRUE)){
+    message('Subordinate clustering package (DatabionicSwarm) is missing. No computations are performed.
+            Please install the package which is defined in "Suggests".')
+    return(list(Cls = Cls,
+    Object = "Subordinate clustering package (DatabionicSwarm) is missing.
                 Please install the package which is defined in 'Suggests'."
       )
     )
@@ -50,7 +47,7 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
 
   message("Operator: Computing nonlinear projection using the swarm.")
   if(is.null(DistancesMethod)){
-    if (isSymmetric(unname(DataOrDistances))) {
+    if(isSymmetric(unname(DataOrDistances))){
       DataDists = DataOrDistances
       if(requireNamespace("ProjectionBasedClustering",quietly = TRUE)){
         if(isTRUE(PlotMap)){
@@ -105,17 +102,20 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
   if(ClusterNo==0){
     #if(isTRUE(PlotTree)){
       message("Operator: Generating dendrogram.")
-      Cls=DatabionicSwarm::DBSclustering(1,DataOrDistance = DataOrDistances,BestMatches =TwoD_Points,LC = LC,
-                                         StructureType = StructureType,PlotIt = PlotTree)
+      Cls=DatabionicSwarm::DBSclustering(k = 1,DataOrDistance = DataOrDistances,BestMatches =TwoD_Points,
+                                         LC = LC, StructureType = StructureType,PlotIt = PlotTree)
     #}#end isTRUE(PlotTree
 
      if(!is.null(DistancesMethod))    
        DataPoints=ProjectionBasedClustering::MDS(DataDists,OutputDimension = nrow(DataDists))$ProjectedPoints
 
-       generalizedUmatrix=DatabionicSwarm::GeneratePswarmVisualization(Data = DataPoints,ProjectedPoints = proj$ProjectedPoints,LC = proj$LC)
+       generalizedUmatrix=DatabionicSwarm::GeneratePswarmVisualization(Data = DataPoints,
+                                                                       ProjectedPoints = proj$ProjectedPoints,
+                                                                       LC = proj$LC)
      if (requireNamespace('GeneralizedUmatrix',quietly = TRUE)){
        message("Operator: Generating topview of topographic map of high-dimensional structures.")
-       out=GeneralizedUmatrix::TopviewTopographicMap(GeneralizedUmatrix = generalizedUmatrix$Umatrix,BestMatchingUnits = generalizedUmatrix$Bestmatches)
+       out=GeneralizedUmatrix::TopviewTopographicMap(GeneralizedUmatrix = generalizedUmatrix$Umatrix,
+                                                     BestMatchingUnits = generalizedUmatrix$Bestmatches)
        print(out)
       }else{
        message(
@@ -128,10 +128,10 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
     message("Operator: Clustering the dataset.")
  
     if(is.null(DistancesMethod)){
-      Cls=DatabionicSwarm::DBSclustering(ClusterNo,DataOrDistance = DataOrDistances,BestMatches =TwoD_Points,LC = LC,
+      Cls=DatabionicSwarm::DBSclustering(ClusterNo,DataOrDistance = DataOrDistances,BestMatches =TwoD_Points, LC = LC,
                         StructureType = StructureType,PlotIt = PlotTree)
     }else{
-      Cls=DatabionicSwarm::DBSclustering(ClusterNo,DataOrDistance = DataDists,BestMatches =TwoD_Points,LC = LC,
+      Cls=DatabionicSwarm::DBSclustering(ClusterNo,DataOrDistance = DataDists,BestMatches =TwoD_Points, LC = LC,
                         StructureType = StructureType,PlotIt = PlotTree)
     }#end is.null(DistancesMethod))
     Cls=ClusterRename(Cls,DataOrDistances)
@@ -142,8 +142,7 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
       generalizedUmatrix = DatabionicSwarm::GeneratePswarmVisualization(
         Data = DataPoints,
         ProjectedPoints = proj$ProjectedPoints,
-        LC = proj$LC
-      )
+        LC = proj$LC)
       GeneralizedUmatrix = generalizedUmatrix$Umatrix
       BestMatchingUnits = generalizedUmatrix$Bestmatches
       if (requireNamespace('GeneralizedUmatrix', quietly = TRUE)) {
@@ -165,5 +164,8 @@ DatabionicSwarmClustering=function(DataOrDistances,ClusterNo=0,StructureType=TRU
 	  ClusterPlotMDS(DataOrDistances,Cls)
   }
   
-  return(list(Cls=Cls,Object=list(Projection=proj,GeneralizedUmatrixOfSwarm=generalizedUmatrix,Call=match.call(),DataPoints=DataPoints)))
+  return(list(Cls=Cls,Object=list(Projection=proj,
+                                  GeneralizedUmatrixOfSwarm=generalizedUmatrix,
+                                  Call=match.call(),
+                                  DataPoints=DataPoints)))
 }
